function [DATA,File]=Filter(DATA,File,BPFfc,DECI)
% Apply Band Pass Filter to DATA and decimate DATA. Filter type is Butterworth, 4th order.
%
% Usage : [DATA,File] = Filter(DATA,File,BPFfc,DECI);
%
% *** INPUT Parameters ***
% DATA,File : See ReadWIN.m
% BPFfc     : Band Pass Filter Parameter [LowCut(Hz) HighCut(Hz)] 
%           : Ex) [10  20] (Band Pass Filter)
%           : Ex) [NaN 20] (Low  Pass Filter)
%           : Ex) [10 NaN] (High Pass Filter)
%           : []           (No filter)
% DECI      : Decimation Frequency (Hz). This must be integer fractions of the original sampling frequency. Default is none.
%
% *** UPDATES ***
% 2008/July/18(Fri) : Original by Koji MIYAKAWA
%

% ---------------------------
% Bandpass Filtering
%
if exist('BPFfc','var') && ~isempty(BPFfc) && length(BPFfc) == 2
    Order      = 4;                                        % Filter Parameter
    File.BPFfc = BPFfc;
    PassBand(:,1) = BPFfc(1)./File.SmpFrq'.*2;
    PassBand(:,2) = BPFfc(2)./File.SmpFrq'.*2;
    for ii = 1:size(PassBand,1)
        if isnan(BPFfc(1)) || BPFfc(1) == 0                 % Low Pass Filter
            if PassBand(ii,2)>0 && PassBand(ii,2)<1
                fprintf('Low Pass Filter is applied to the %GHz data (fc = %GHz).\n',File.SmpFrq(ii),BPFfc(2));
                [Bb,Ab]  = butter(Order,PassBand(ii,2),'low');
            else
               continue;
            end
        elseif isnan(BPFfc(2))                              % High Pass Filter
            if PassBand(ii,1)>0 && PassBand(ii,1)<1
                fprintf('High Pass Filter is applied to the %GHz data (fc = %GHz).\n',File.SmpFrq(ii),BPFfc(1));
                [Bb,Ab]  = butter(Order,PassBand(ii,1),'high');
            else
               continue;
            end
        else                                                % Bandpass Filter
            if PassBand(ii,1)>0 && PassBand(ii,1)<1
                fprintf('Band Pass Filter is applied to the %GHz data (fc = %GHz - %GHz).\n',File.SmpFrq(ii),BPFfc);
                [Bb,Ab]  = butter(Order,PassBand(ii,:));
            else
               continue;
            end
        end
        for jj = 1:size(DATA{ii},2)
            DATA{ii}(:,jj) = filtfilt(Bb,Ab,DATA{ii}(:,jj)-mean(DATA{ii}(:,jj)));   % Zero Phase Delay
        end
    end
else
%   fprintf('No Filter is applied.\n\n');
end

% ---------------------------
% Decimation
%
if exist('DECI','var') && ~isempty(DECI)
    for ii = 1:length(DATA)
        if rem(ceil(File.SmpFrq(ii))/DECI,1) == 0
            DATA{ii} = DATA{ii}(1:ceil(File.SmpFrq(ii))/DECI:end,:);
            File.OrgSmpFrq(ii) = File.SmpFrq(ii);
            File.SmpFrq(ii)    = DECI;
            fprintf('DATA is decimated to %3GHz on the %GHz data.\n',DECI,File.OrgSmpFrq(ii));
        end
    end
end

% ---------------------------
% Add Carriage Return
%
if (exist('BPFfc','var') && ~isempty(BPFfc) && length(BPFfc) == 2) || (exist('DECI','var') && ~isempty(DECI))
    fprintf('\n');
end
