function Plot_PSDs(File,ChTbl,PSD,XScale,XLIM,YLIM,UsgsNM,MvAvg)
% Plot multiple PSDs in multiple axeses
%
% Usage : Plot_PSDs(File,ChTbl,PSD,XScale,XLIM,YLIM,MvAvg)
%
% *** INPUT Parameters ***
% File,ChTbl,PSD : See ReadWIN.m
% XScale : Xaxis Scale of the PSD plot. 'log' or 'linear' or '' or []. Default is 'linear'.
% XLIM   : Xaxis limits (frequency) of the PSD plot. [Fmin Fmax] or [] (Display whole range).
% YLIM   : Yaxis limits of the PSD plot. [Ymin Ymax] or [] (Display whole range)
% UsgsNM : Plot USGS noise model(NHNM/NLNM) on the PSD axeses. 'Yes' or 'No' or ''. Default is 'Yes'
% MvAvg  : Moving average for smoothing the PSD plot. Default is 1 (do not apply moving average).
%
% *** UPDATES ***
% 2008/July/18(Fri) : Original by Koji MIYAKAWA
%

clf;
% ---------------------------
% Parameter Initialization
%
RECT  = [0.08 0.05 0.88 0.94];      % RECT = get(gcf,'DefaultAxesPosition');
NumCh = sum(File.NumChs);
ha    = zeros(NumCh,1);
if ~exist('XLIM','var') || isempty(XLIM)        % Check XLIM
    for ii = 1:length(File.NumChs)
        XLIM(ii,:) = [PSD{ii}.F(1) PSD{ii}.F(end)];
    end
else
    XLIM = ones(length(File.NumChs),1)*XLIM;
end
if ~exist('YLIM','var') || isempty(YLIM)        % Check YLIM
    for ii = 1:length(File.NumChs)
        YLIM(ii,:) = [min(min(PSD{ii}.PSD(5:end,:))) max(max(PSD{ii}.PSD(5:end,:)))]; % Omit PSD.PSD(1:4,:) to remove the effect of its DC offset
    end
%   YLIM = ones(length(File.NumChs),1)*[min(YLIM(:,1)),max(YLIM(:,2))];	% Set the same YLIM in all axeses
else
    YLIM = ones(length(File.NumChs),1)*YLIM;
end
if exist('MvAvg','var') && ~isempty(MvAvg) && MvAvg > 1 % Check MvAvg
    if ~rem(MvAvg,2)
        error('MvAvg Error : Number of data points for moving average must be odd number.');
    else
        a = 1;
        b = ones(1,MvAvg)/MvAvg;
        for ii = 1:length(PSD)
            for jj = 1:size(PSD{ii},2)
                PSD{ii}.PSD(:,jj) = filtfilt(b,a,PSD{ii}.PSD(:,jj));
            end
        end
    end
else
    MvAvg = 1;
end
if exist('File','var') && isfield(File,'BPFfc') && ~isempty(File.BPFfc) % Check File.BPFfc
    if isnan(File.BPFfc(1))
        FiltStr = ['LPF=',num2str(File.BPFfc(2)),'Hz'];
    elseif isnan(File.BPFfc(2))
        FiltStr = ['HPF=',num2str(File.BPFfc(1)),'Hz'];
    else
        FiltStr = ['BPF=',num2str(File.BPFfc(1)),'-',num2str(File.BPFfc(2)),'Hz'];
    end
else
    FiltStr = 'No Filter';
end
if isempty(File.PSD_WndwSec)                    % Check File.PSD_WndwSec
    PSD_WndwSec = 1/(PSD{1}.F(2)-PSD{1}.F(1));
else
    PSD_WndwSec = File.PSD_WndwSec;
end
if isempty(File.PSD_OvlpPct)                    % Check File.PSD_OvlpPct
    PSD_OvlpPct = 50;                           % Empty variable means default(50%)
else
    PSD_OvlpPct = File.PSD_OvlpPct;
end

% ---------------------------
% Plot
%
for ii = 1:NumCh
    TarCol = File.ColChs(ii);
    TarRow = File.RowChs(ii);
    ha(ii) = axes('Position',[RECT(1,1)+0.015 RECT(1,2)+RECT(1,4)*(NumCh-ii)/NumCh RECT(1,3) RECT(1,4)/NumCh*0.93]);
    plot(PSD{TarCol}.F,PSD{TarCol}.PSD(:,TarRow)); grid on;
    ylabel(sprintf('PSD/(%s)^2/Hz',ChTbl.Unit{ii}));
    TEXT = {sprintf('\\bf%s-%s(%s)\\rm',strrep(ChTbl.StnNm{ii},'_','\_'),ChTbl.Comp{ii},ChTbl.ChID{ii})};
    if exist('XScale','var') && strcmpi(XScale,'log')   % Set Xscale 'log'
        if exist('UsgsNM','var') && ~strcmpi(UsgsNM,'No')
            if strcmpi(ChTbl.Unit{ii},'m/s/s')
                NMch = 4;
            elseif strcmpi(ChTbl.Unit{ii},'m/s')
                NMch = 3;
            elseif strcmpi(ChTbl.Unit{ii},'m')
                NMch = 2;
            else
                NMch = [];
            end
            if ~isempty(NMch)
                if ~exist('NHNM','var')
                    load('USGS_NLNM_NHNM.mat');             % Load USGS_NLNM_NHNM
                end
                hold on;
                plot(NHNM(:,1),NHNM(:,NMch),'r--',NLNM(:,1),NLNM(:,NMch),'k--');
                hold off;
                TEXT{2} = 'USGS-NHNM';
                TEXT{3} = 'USGS-NLNM';
            end
        end
        if XLIM(TarCol,1) == 0
            if XLIM(TarCol,2) < 0.01
                XLIM(TarCol,1) = 10^floor(log10(XLIM(TarCol,2)));
            else
                XLIM(TarCol,1) = 0.01;     % Avoid log10(0)
            end
        end
        xlim(XLIM(TarCol,:));
        set(ha(ii),'XScale','log','XTick',10.^(fix(log10(XLIM(TarCol,1))):floor(log10(XLIM(TarCol,2)))));    % Set XTick to every figure in log scale
    else
        xlim(XLIM(TarCol,:));
        DoubleTick(ha(ii),'x');    % Double the number of the XTick
    end
    ylim(YLIM(TarCol,:));
    set(ha(ii),'YScale','log','YTick',10.^(fix(log10(YLIM(TarCol,1))):floor(log10(YLIM(TarCol,2)))));        % Set YTick to every figure in log scale
    if ii == NumCh
        xlabel(sprintf('Frequency/Hz (%s, Window=%Gsec, Overlap=%G%%, MovingAvg=%dpts)',FiltStr,PSD_WndwSec,PSD_OvlpPct,MvAvg));
    end
    legend(ha(ii),TEXT);
end
set(ha,'Box','on');

% ---------------------------
% Text Information
%
CurAxes = gca;
axes('Position',[0 0 1 1],'Visible','off');
if File.RecSec < 3*3600                         % Set 'minute' if the data length is less than 3 hours
    DUR = [num2str(round(File.RecSec/60*10)/10),'min'];
else
    DUR = [num2str(round(File.RecSec/3600*10)/10),'hr'];
end
TITLE = sprintf('%s %s - %s (%s)',File.StDate,File.StTime,File.EdTime,DUR);
text(0.5,0.998,TITLE,'HorizontalAlignment','center','VerticalAlignment','top','FontSize',12,'FontWeight','bold');
set(gcf,'CurrentAxes',CurAxes);

% ===========================
function DoubleTick(ha,Axis)
% Double the number of the Tick
%
if strcmpi(Axis,'x')
    LIMstr  = 'XLim';
    TICKstr = 'XTick';
elseif strcmpi(Axis,'y')
    LIMstr  = 'YLim';
    TICKstr = 'YTick';
else
    return
end
LIM   = get(ha,LIMstr);
TICK  = get(ha,TICKstr);
dTICK = (TICK(2)-TICK(1))/2;
if LIM(2) > TICK(end)+dTICK
    TICK(end) = TICK(end)+dTICK;
end
set(ha,TICKstr',TICK(1):dTICK:TICK(end));
